# -*- encoding: utf-8 -*-

module Rumoji
  class Emoji
    require 'set'
    
    attr_reader :name

    def initialize(string, symbols, name)
      @codepoints = string.codepoints
      @cheat_codes = [symbols].flatten
      @name = name
    end

    def code
      @cheat_codes.first
    end

    def include?(symbol)
      @cheat_codes.include? symbol.to_sym
    end

    def to_s
      @codepoints.to_a.pack("U*")
    end

    def hash
      code.hash
    end

    def hex
      @codepoints.map{|point| point.to_s(16).upcase }.join
    end

    PEOPLE = Set[
      self.new("\u{1F604}", [:smile], "SMILING FACE WITH OPEN MOUTH AND SMILING EYES"),
      self.new("\u{1F606}", [:laughing], ""),
      self.new("\u{1F60A}", [:blush], ""),
      self.new("\u{1F603}", [:smiley], ""),
      self.new("\u{263A}", [:relaxed], ""),
      self.new("\u{1F60F}", [:smirk], ""),
      self.new("\u{1F60D}", [:heart_eyes], ""),
      self.new("\u{1F618}", [:kissing_heart], ""),
      self.new("\u{1F61A}", [:kissing_closed_eyes], ""),
      self.new("\u{1F633}", [:flushed], ""),
      self.new("\u{1F625}", [:relieved], ""),
      self.new("\u{1F60C}", [:satisfied], ""),
      self.new("\u{1F601}", [:grin], ""),
      self.new("\u{1F609}", [:wink], ""),
      self.new("\u{1F61C}", [:wink2, :stuck_out_tongue_winking_eye], ""),
      self.new("\u{1F61D}", [:stuck_out_tongue_closed_eyes], ""),
      self.new("\u{1F600}", [:grinning], ""),
      self.new("\u{1F617}", [:kissing], ""),
      self.new("\u{1F619}", [:kissing_smiling_eyes], ""),
      self.new("\u{1F61B}", [:stuck_out_tongue], ""),
      self.new("\u{1F634}", [:sleeping], ""),
      self.new("\u{1F61F}", [:worried], ""),
      self.new("\u{1F626}", [:frowning], ""),
      self.new("\u{1F627}", [:anguished], ""),
      self.new("\u{1F62E}", [:open_mouth], ""),
      self.new("\u{1F62C}", [:grimacing], ""),
      self.new("\u{1F615}", [:confused], ""),
      self.new("\u{1F62F}", [:hushed], ""),
      self.new("\u{1F611}", [:expressionless], ""),
      self.new("\u{1F612}", [:unamused], ""),
      self.new("\u{1F605}", [:sweat_smile], ""),
      self.new("\u{1F613}", [:sweat], ""),
      self.new("\u{1F629}", [:weary], ""),
      self.new("\u{1F614}", [:pensive], ""),
      self.new("\u{1F61E}", [:dissapointed], ""),
      self.new("\u{1F616}", [:confounded], ""),
      self.new("\u{1F628}", [:fearful], ""),
      self.new("\u{1F630}", [:cold_sweat], ""),
      self.new("\u{1F623}", [:persevere], ""),
      self.new("\u{1F622}", [:cry], ""),
      self.new("\u{1F62D}", [:sob], ""),
      self.new("\u{1F602}", [:joy], ""),
      self.new("\u{1F632}", [:astonished], ""),
      self.new("\u{1F631}", [:scream], ""),
      self.new("\u{1F62B}", [:tired_face], ""),
      self.new("\u{1F620}", [:angry], ""),
      self.new("\u{1F621}", [:rage], ""),
      self.new("\u{1F624}", [:triumph], ""),
      self.new("\u{1F62A}", [:sleepy], ""),
      self.new("\u{1F60B}", [:yum], ""),
      self.new("\u{1F637}", [:mask], ""),
      self.new("\u{1F60E}", [:sunglasses], ""),
      self.new("\u{1F635}", [:dizzy_face], ""),
      self.new("\u{1F47F}", [:imp], ""),
      self.new("\u{1F608}", [:smiling_imp], ""),
      self.new("\u{1F610}", [:neutral_face], ""),
      self.new("\u{1F636}", [:no_mouth], ""),
      self.new("\u{1F607}", [:innocent], ""),
      self.new("\u{1F47D}", [:alien], ""),
      self.new("\u{1F49B}", [:yellow_heart], ""),
      self.new("\u{1F499}", [:blue_heart], ""),
      self.new("\u{1F49C}", [:purple_heart], ""),
      self.new("\u{2764}", [:heart], ""),
      self.new("\u{1F49A}", [:green_heart], ""),
      self.new("\u{1F494}", [:broken_heart], ""),
      self.new("\u{1F493}", [:heartbeat], ""),
      self.new("\u{1F497}", [:heartpulse], ""),
      self.new("\u{1F495}", [:two_hearts], ""),
      self.new("\u{1F49E}", [:revolving_hearts], ""),
      self.new("\u{1F498}", [:cupid], ""),
      self.new("\u{1F496}", [:sparkling_heart], ""),
      self.new("\u{2728}", [:sparkles], ""),
      self.new("\u{2B50}", [:star], ""),
      self.new("\u{1F31F}", [:star2], ""),
      self.new("\u{1F4AB}", [:dizzy], ""),
      self.new("\u{1F4A5}", [:boom, :collision], ""),
      self.new("\u{1F4A2}", [:anger], ""),
      self.new("\u{2757}", [:exclamation], ""),
      self.new("\u{2753}", [:question], ""),
      self.new("\u{2755}", [:grey_exclamation], ""),
      self.new("\u{2754}", [:grey_question], ""),
      self.new("\u{1F4A4}", [:zzz], ""),
      self.new("\u{1F4A8}", [:dash], ""),
      self.new("\u{1F4A6}", [:sweat_drops], ""),
      self.new("\u{1F3B6}", [:notes], ""),
      self.new("\u{1F3B5}", [:musical_note], ""),
      self.new("\u{1F525}", [:fire], ""),
      # Poop
      self.new("\u{1F4A9}", [:hankey, :poop, :shit], "PILE OF POO"),
      self.new("\u{1F44D}", [:thumbsup], ""),
      self.new("\u{1F44E}", [:thumbsdown], ""),
      self.new("\u{1F44C}", [:ok_hand], ""),
      self.new("\u{1F44A}", [:punch, :facepunch], ""),
      self.new("\u{270A}", [:fist], ""),
      self.new("\u{270C}", [:v], ""),
      self.new("\u{1F44B}", [:wave], ""),
      self.new("\u{270B}", [:hand], ""),
      self.new("\u{1F450}", [:open_hands], ""),
      self.new("\u{261D}", [:point_up], ""),
      self.new("\u{1F447}", [:point_down], ""),
      self.new("\u{1F448}", [:point_left], ""),
      self.new("\u{1F449}", [:point_right], ""),
      self.new("\u{1F64C}", [:raised_hands], ""),
      self.new("\u{1F64F}", [:pray], ""),
      self.new("\u{1F446}", [:point_up_2], ""),
      self.new("\u{1F44F}", [:clap], ""),
      self.new("\u{1F4AA}", [:muscle], ""),
      self.new("\u{1F6B6}", [:walking], ""),
      self.new("\u{1F3C3}", [:runner, :running], "RUNNER"),
      self.new("\u{1F46B}", [:couple], "MAN AND WOMAN HOLDING HANDS"),
      self.new("\u{1F46A}", [:family], "FAMILY"),
      self.new("\u{1F46C}", [:two_men_holding_hands], "TWO MEN HOLDING HANDS"),
      self.new("\u{1F46D}", [:two_women_holding_hands], "TWO WOMEN HOLDING HANDS"),
      self.new("\u{1F483}", [:dancer], ""),
      self.new("\u{1F46F}", [:dancers], ""),
      self.new("\u{1F646}", [:ok_woman], ""),
      self.new("\u{1F645}", [:no_good], ""),
      self.new("\u{1F481}", [:information_desk_person], ""),
      self.new("\u{1F64B}", [:raised_hand], ""),
      self.new("\u{1F470}", [:bride_with_veil], ""),
      self.new("\u{1F64E}", [:person_with_pouting_face], ""),
      self.new("\u{1F64D}", [:person_frowning], ""),
      self.new("\u{1F647}", [:bow], ""),
      self.new("\u{1F48F}", [:couplekiss], ""),
      self.new("\u{1F491}", [:couple_with_heart], ""),
      self.new("\u{1F486}", [:massage], ""),
      self.new("\u{1F487}", [:haircut], ""),
      self.new("\u{1F485}", [:nail_care], ""),
      self.new("\u{1F466}", [:boy], ""),
      self.new("\u{1F467}", [:girl], ""),
      self.new("\u{1F469}", [:woman], ""),
      self.new("\u{1F468}", [:man], ""),
      self.new("\u{1F476}", [:baby], ""),
      self.new("\u{1F475}", [:older_woman], ""),
      self.new("\u{1F474}", [:older_man], ""),
      self.new("\u{1F471}", [:person_with_blond_hair], ""),
      self.new("\u{1F472}", [:man_with_gua_pi_mao], ""),
      self.new("\u{1F473}", [:man_with_turban], ""),
      self.new("\u{1F477}", [:construction_worker], ""),
      self.new("\u{1F46E}", [:cop], ""),
      self.new("\u{1F47C}", [:angel], ""),
      self.new("\u{1F478}", [:princess], ""),
      # Cats
      self.new("\u{1F63A}", [:smiley_cat], ""),
      self.new("\u{1F638}", [:smile_cat], ""),
      self.new("\u{1F63B}", [:heart_eyes_cat], ""),
      self.new("\u{1F63D}", [:kissing_cat], ""),
      self.new("\u{1F63C}", [:smirk_cat], ""),
      self.new("\u{1F640}", [:scream_cat], ""),
      self.new("\u{1F63F}", [:crying_cat_face], ""),
      self.new("\u{1F639}", [:joy_cat], ""),
      self.new("\u{1F63E}", [:pouting_cat], ""),
      self.new("\u{1F479}", [:japanese_ogre], ""),
      self.new("\u{1F47A}", [:japanese_goblin], ""),
      # Monkeys
      self.new("\u{1F648}", [:see_no_evil], ""),
      self.new("\u{1F649}", [:hear_no_evil, :speak_no_evil], ""),
      self.new("\u{1F482}", [:guardsman], ""),
      self.new("\u{1F480}", [:skull], ""),
      self.new("\u{1F463}", [:feet], ""),
      self.new("\u{1F444}", [:lips], ""),
      self.new("\u{1F48B}", [:kiss], ""),
      self.new("\u{1F4A7}", [:droplet], ""),
      self.new("\u{1F442}", [:ear], ""),
      self.new("\u{1F440}", [:eyes], ""),
      self.new("\u{1F443}", [:nose], ""),
      self.new("\u{1F445}", [:tongue], ""),
      self.new("\u{1F48C}", [:love_letter], ""),
      self.new("\u{1F464}", [:bust_in_silhouette], ""),
      self.new("\u{1F465}", [:busts_in_silhouette], ""),
      self.new("\u{1F4AC}", [:speech_balloon], ""),
      self.new("\u{1F4AD}", [:thought_balloon], ""),
    ]

    NATURE = Set[
      self.new("\u{2600}", [:sunny], ""),
      self.new("\u{2614}", [:umbrella], ""),
      self.new("\u{2601}", [:cloud], ""),
      self.new("\u{2744}", [:snowflake], ""),
      self.new("\u{26C4}", [:snowman], ""),
      self.new("\u{26A1}", [:zap], ""),
      self.new("\u{1F300}", [:cyclone], ""),
      self.new("\u{1F301}", [:foggy], ""),
      self.new("\u{1F30A}", [:ocean], ""),
      # Animals
      self.new("\u{1F431}", [:cat], ""),
      self.new("\u{1F436}", [:dog], ""),
      self.new("\u{1F42D}", [:mouse], ""),
      self.new("\u{1F439}", [:hamster], ""),
      self.new("\u{1F430}", [:rabbit], ""),
      self.new("\u{1F43A}", [:wolf], ""),
      self.new("\u{1F438}", [:frog], ""),
      self.new("\u{1F42F}", [:tiger], ""),
      self.new("\u{1F428}", [:koala], ""),
      self.new("\u{1F43B}", [:bear], ""),
      self.new("\u{1F437}", [:pig], ""),
      self.new("\u{1F43D}", [:pig_nose], ""),
      self.new("\u{1F42E}", [:cow], ""),
      self.new("\u{1F417}", [:boar], ""),
      self.new("\u{1F435}", [:monkey_face], ""),
      self.new("\u{1F412}", [:monkey], ""),
      self.new("\u{1F434}", [:horse], ""),
      self.new("\u{1F40E}", [:racehorse], ""),
      self.new("\u{1F42B}", [:camel], ""),
      self.new("\u{1F411}", [:sheep], ""),
      self.new("\u{1F418}", [:elephant], ""),
      self.new("\u{1F43C}", [:panda_face], ""),
      self.new("\u{1F40D}", [:snake], ""),
      self.new("\u{1F426}", [:bird], ""),
      self.new("\u{1F424}", [:baby_chick], ""),
      self.new("\u{1F425}", [:hatched_chick], ""),
      self.new("\u{1F423}", [:hatching_chick], ""),
      self.new("\u{1F414}", [:chicken], ""),
      self.new("\u{1F427}", [:penguin], ""),
      self.new("\u{1F422}", [:turtle], ""),
      self.new("\u{1F41B}", [:bug], ""),
      self.new("\u{1F41D}", [:honeybee], ""),
      self.new("\u{1F41C}", [:ant], ""),
      self.new("\u{1F41E}", [:beetle], ""),
      self.new("\u{1F40C}", [:snail], ""),
      self.new("\u{1F419}", [:octopus], ""),
      self.new("\u{1F420}", [:tropical_fish], ""),
      self.new("\u{1F41F}", [:fish], ""),
      self.new("\u{1F433}", [:whale], ""),
      self.new("\u{1F40B}", [:whale2], ""),
      self.new("\u{1F42C}", [:dolphin], ""),
      self.new("\u{1F404}", [:cow2], ""),
      self.new("\u{1F40F}", [:ram], ""),
      self.new("\u{1F400}", [:rat], ""),
      self.new("\u{1F403}", [:water_buffalo], ""),
      self.new("\u{1F405}", [:tiger2], ""),
      self.new("\u{1F407}", [:rabbit2], ""),
      self.new("\u{1F409}", [:dragon], ""),
      self.new("\u{1F410}", [:goat], ""),
      self.new("\u{1F413}", [:rooster], ""),
      self.new("\u{1F415}", [:dog2], ""),
      self.new("\u{1F416}", [:pig2], ""),
      self.new("\u{1F401}", [:mouse2], ""),
      self.new("\u{1F402}", [:ox], ""),
      self.new("\u{1F432}", [:dragon_face], ""),
      self.new("\u{1F421}", [:blowfish], ""),
      self.new("\u{1F40A}", [:crocodile], ""),
      self.new("\u{1F42A}", [:dromedary_camel], ""),
      self.new("\u{1F406}", [:leopard], ""),
      self.new("\u{1F408}", [:cat2], ""),
      self.new("\u{1F429}", [:poodle], ""),
      self.new("\u{1F43E}", [:paw_prints], ""),
      # Flowers
      self.new("\u{1F490}", [:bouquet], ""),
      self.new("\u{1F338}", [:cherry_blossom], ""),
      self.new("\u{1F337}", [:tulip], ""),
      self.new("\u{1F340}", [:four_leaf_clover], ""),
      self.new("\u{1F339}", [:rose], ""),
      self.new("\u{1F33B}", [:sunflower], ""),
      self.new("\u{1F33A}", [:hibiscus], ""),
      self.new("\u{1F341}", [:maple_leaf], ""),
      self.new("\u{1F343}", [:leaves], ""),
      self.new("\u{1F342}", [:fallen_leaf], ""),
      self.new("\u{1F33F}", [:herb], ""),
      self.new("\u{1F344}", [:mushroom], ""),
      self.new("\u{1F335}", [:cactus], ""),
      self.new("\u{1F334}", [:palm_tree], ""),
      self.new("\u{1F332}", [:evergreen_tree], ""),
      self.new("\u{1F333}", [:deciduous_tree], ""),
      self.new("\u{1F330}", [:chestnut], ""),
      self.new("\u{1F331}", [:seedling], ""),
      self.new("\u{1F33C}", [:blossum], ""),
      self.new("\u{1F33E}", [:ear_of_rice], ""),
      self.new("\u{1F41A}", [:shell], ""),
      self.new("\u{1F310}", [:globe_with_meridians], ""),
      # Moon
      self.new("\u{1F31E}", [:sun_with_face], ""),
      self.new("\u{1F31D}", [:full_moon_with_face], ""),
      self.new("\u{1F31A}", [:new_moon_with_face], ""),
      self.new("\u{1F311}", [:new_moon], ""),
      self.new("\u{1F312}", [:waxing_crescent_moon], ""),
      self.new("\u{1F313}", [:first_quarter_moon], ""),
      self.new("\u{1F314}", [:waxing_gibbous_moon], ""),
      self.new("\u{1F315}", [:full_moon], ""),
      self.new("\u{1F316}", [:waning_gibbous_moon], ""),
      self.new("\u{1F317}", [:last_quarter_moon], ""),
      self.new("\u{1F318}", [:waning_crescent_moon], ""),
      self.new("\u{1F31C}", [:last_quarter_moon_with_face], ""),
      self.new("\u{1F31B}", [:first_quarter_moon_with_face], ""),
      self.new("\u{1F319}", [:moon], ""),
      self.new("\u{1F30D}", [:earth_africa], ""),
      self.new("\u{1F30E}", [:earth_americas], ""),
      self.new("\u{1F30F}", [:earth_asia], ""),
      self.new("\u{1F30B}", [:volcano], ""),
      self.new("\u{1F30C}", [:milky_way], ""),
      self.new("\u{26C5}", [:partly_sunny], "")
    ]

    ALL = PEOPLE | NATURE

    def self.find(symbol)
      ALL.find {|emoji| emoji.include? symbol }
    end

    def self.find_by_string(string)
      ALL.find {|emoji| emoji.to_s == string }
    end

    def self.find_by_codepoint(codepoint)
      ALL.find {|emoji| emoji.hex == codepoint.to_s(16).upcase }
    end
  end
end
